//***********************************************
//*												*
//*           	  Exercise 3-2	    			*
//*												*
//***********************************************

#include <p18f452.h>
#include <adc.h>
#include <pwm.h>
#include <timers.h>

void InitializeAD(void);
void InitializePWM2(void);
void InitializeTMR2(void);
void InitializePORT(void);

unsigned int AD_Result;

void main(void)
{
	InitializePORT( );
	InitializePWM2( );
	InitializeAD( );		
	InitializeTMR2( );

	while(1)
	{
		ConvertADC();							// Convert A/D
		while(BusyADC());						// Waiting A/D until done
		AD_Result=ReadADC();					// Get 10 bits A/D result
		SetDCPWM2(AD_Result);					// Put 10 bits A/D result to PWM1
		PORTD=(unsigned char)(AD_Result>>2); 	// Right shift 2 bits, put MSB on LEDs				
	}					 
}

//************************************************
//* Function: InitializeAD                       *
//*        - Fosc/32 clock source                 *
//*        - Right justified                     *
//*        - 1 analog channels, 0 references     *
//*        - Channel 0                           *
//*        - Disable A/D interrupt               *
//************************************************
void InitializeAD(void)
{
	OpenADC(ADC_FOSC_32 & ADC_RIGHT_JUST & ADC_1ANA_0REF,
			ADC_CH0 & ADC_INT_OFF);
}
//************************************************
//* Function: InitializePWM2                     *
//*   - Initialize the CCP2 Module as PWM output *
//*     for controlling motor speed              *
//*     (see p160 MPLAB-CXX Reference Guide)     *
//*        - Set the period to FFh               *
//*        - Make RC1 an output                  *
//*        - Configure CCP2 for PWM              *
//************************************************
void InitializePWM2(void)
{
	TRISCbits.TRISC1=0;	//* setup RC1 for PWM output
	OpenPWM2(0xff); 	//* set period to 0xFF
}


//************************************************
//* Function: InitializeTimer2                   *
//*   - Initialize Timer2 for use with CCP1 PWM  *
//*        - 1:16 Postscale                      *
//*        - 1:4 Prescale                        *
//*        - Enable the timer                    *
//************************************************
void InitializeTMR2(void)
{
	OpenTimer2(T2_PS_1_4 & T2_POST_1_16);
}
//************************************************
//* Function: InitializePORT                     *
//*   - Make PORTD all outputs                   *
//*   - Turn 0ff all the LED's                   *
//************************************************
void InitializePORT(void)
{
	TRISD=0;
	PORTD=0x00;
}

	



