//***********************************************
//*												*
//*           	  Exercise 5-3					*
//*												*
//***********************************************
/* Include Header files  */

#include <p18f452.h>
#include <adc.h>
#include <pwm.h>
#include <timers.h>
#include <delays.h>
#include <usart.h>
#include <stdlib.h>
#include "p18LCD.h"

/* Declare the Function Prototype */
void InitializeAD(void);
void InitializePWM1(void);
void InitializeTMR2(void);
void InitializeUSART(void);
void InitializeINTs(void);
void InitializePORT(void);
void isr_high_direct(void);
void isr_high(void);
void Display_Terminal(const rom far char *data);
void Send_CR_LF(void);
void Speed_LCD(void);
void Motor_Ctrl(void);

/* define the LCD_MSG1[] in program memory at 0x1000 */
//***************************************************** 
#pragma romdata My_romdata=0x1000
const rom far unsigned char LCD_MSG1[]="  C18 Workshop ";
const rom far unsigned char LCD_MSG2[]="A/D Result = ";

const rom far char Disp_Msg1[]="MPLAB-C18 Workshop";
const rom far char Disp_Msg2[]="Motor is Running !"; 
const rom far char Disp_Msg3[]="Motor is Stopped !";
const rom far char Disp_Msg4[]="Current Motor Speed are : ";
#pragma romdata


/* define following variable in data memory at Access Bank */
//*********************************************************** 
#pragma udata access My_RAM_2
near unsigned int AD_Temp;
near char LCD_Temp;
near unsigned char Rec_Data;
near unsigned char PWM_Ctrl;


union
{
	unsigned int AD_10bit;    // contant is 10-bit A/D Result
    unsigned char AD[2];	  // AD[0] is A/D Low byte, AD[1] is A/D High Byte
} near AD_Result;
#pragma udata

char Disp_ptr[6];
char Hex_ASCII_Code(char Data);

//***********************************************************
/*														   */
/* 						Main 	Program					   */
/*														   */
//***********************************************************


void main(void)
{
	InitializePORT( );
	InitializePWM1( );
	InitializeAD( );		
	InitializeTMR2( );
	OpenLCD( );
	InitializeINTs( );							
	InitializeUSART( );

	Display_Terminal(Disp_Msg1);
	
 	LCD_Set_Cursor(0,0);						// Put LCD Cursor on Line 1
	putrsLCD(LCD_MSG1);

	LCD_Set_Cursor(1,0);						// Put LCD Cursor on Line 2
	putrsLCD(LCD_MSG2);

	while(1)
	{
		ConvertADC();								// Convert A/D
		while(BusyADC());							// Waiting A/D until done
		AD_Temp=ReadADC();							// Get 10 bits A/D result
		AD_Result.AD_10bit=AD_Temp;
		Speed_LCD();								// Display the Speed on LCD with Hex

		if (Rec_Data=='s')							// Received 's' from RS-232
		{
			PWM_Ctrl=1;								// Yes, Set Motor ON !
			Rec_Data=0x00;
			Display_Terminal(Disp_Msg2);			// Send Msg2 to Terminal
		}
		
		if (Rec_Data=='p') 							// Received 'p' from RS-232
		{
			PWM_Ctrl=0;								// Yes, Motor is OFF
			Rec_Data=0x00;
			Display_Terminal(Disp_Msg3);
		}
		if (Rec_Data=='r')							// Received 'r' Read Cmmand from RS-232
		{
			Rec_Data=0x00;
			Display_Terminal(Disp_Msg4);
			itoa(AD_Temp,Disp_ptr);					// Convert A/D result to ASCII string
			putsUSART(Disp_ptr);					// Send the ASCII string to Terminal
		}
		Motor_Ctrl();								// Motor ON/OFF control Function
	}
}
//***********************************************
//*      Motor On/Off and speed control         *
//***********************************************
void Motor_Ctrl(void)
{		 
		if (PWM_Ctrl)
		{
			TRISCbits.TRISC2=0;					// Enable PWM output
			SetDCPWM1(AD_Result.AD_10bit);		// Put 10 bits A/D result to PWM1

//			AD_Result.AD_10bit>>=2; 			// Right shift 2 bits
//			PORTD=AD_Result.AD[0];				// put the MSB to LEDs
		}
		else
			TRISCbits.TRISC2=1;					// Disable PWM output
}
//***********************************************
//*   Display Speed to LCD with 3 Hex Text      *
//***********************************************
void Speed_LCD(void)
{		 
		AD_Result.AD_10bit=AD_Temp;		
		LCD_Set_Cursor(1,12);								// Set LCD cursor to Line 2 with location 13

		LCD_Temp=Hex_ASCII_Code(AD_Result.AD[1] & 0x03);  	// get (b9,b8) to convert to ASCII
		WriteDataLCD(LCD_Temp);						  	// write to location 13 of LCD


		LCD_Temp=Hex_ASCII_Code(AD_Result.AD[0] >> 4);		// get (b7~b4) to ASCII
		WriteDataLCD(LCD_Temp);


		LCD_Temp=Hex_ASCII_Code(AD_Result.AD[0] & 0x0F); 	// get (b3~b0) to ASCII
		WriteDataLCD(LCD_Temp);
			 					 
}

char Hex_ASCII_Code(char Data) 					// call by value & return value
{

	if (Data<0x0a) return (Data+0x30);			
	else return (Data+0x37);
}
//***********************************************
//*   Send a ASCII string to Terminal           *
//***********************************************
void Display_Terminal(const rom far char *data)	
{	
		Send_CR_LF();
		putrsUSART(data);
		Delay10KTCYx(10);
}
void Send_CR_LF(void)
{
		while (BusyUSART());
		WriteUSART('\n');
		while (BusyUSART());
		WriteUSART('\r');
}
//************************************************
//*       #pragma Interrupt Declarations         *
//************************************************
//************************************************
//* Function: isr_high_direct                    *
//*   - Direct execution to the actual           *
//*     high-priority interrupt code.            *
//************************************************
#pragma code isrhighcode = 0x0008

void isr_high_direct(void)	
{  								
     _asm 		//begin in-line assembly
     goto isr_high	//go to isr_high function
     _endasm 		//end in-line assembly
}
#pragma code

//************************************************
//* Function: isr_high(void)                     *     
//*   High priority interrupt will               *
//*   - Received a serial data from RS-232       *
//*     Save the received data to buffer Rec_Data*                  
//************************************************
#pragma interrupt isr_high 

void isr_high(void)
{
	Rec_Data=ReadUSART();				// Get RS-232 data
 	PORTD=Rec_Data;						// Display to LEDs
}								
#pragma code

//************************************************
//* Function: Initialize Interrupts              *
//*   - Reset All Interrupr                      *
//*   - Enable HIgh Priority Interrupt           *
//*   - Disable Low Priority Interrupt           *
//************************************************
void InitializeINTs(void) 
{
 	PIR1=0; 						//Clear all Interrupt setting & Flag
	PIR2=0;
	INTCON=0;
	INTCON2=0;
	INTCON3=0;
	PIE1=0;
	PIE2=0;

	RCONbits.IPEN=1;     			//Enable priority interrupts

	INTCONbits.GIEL=0;				// Disable Low Priority Interrupt
	INTCONbits.GIEH=1;				// Enalbe high priority Interrupt
}
//************************************************
//* Function: Initialize USART                   *
//*   - Initialize the RS-232 communication      *
//*   - Disable Interrupr for TxD                *
//*   - Enable Interrupt fro Rxd                 *
//*   - Format: 9600, N, 8, 1                    *
//************************************************
void InitializeUSART(void)
{
	OpenUSART( USART_TX_INT_OFF 			// Set TXSTA Reg. =0b00100100 
		 & USART_RX_INT_ON			// Set RCSTA Reg. =0b10010000 
		 & USART_ASYNCH_MODE
		 & USART_EIGHT_BIT
		 & USART_CONT_RX
		 & USART_BRGH_HIGH
		 ,25);					// Set SPBRG=25, Baud Rate = 9600

	RCSTAbits.SPEN=1;
	TRISCbits.TRISC7=1;				// Set input for RXD
	TRISCbits.TRISC6=0;				// Set output for TXD
	PIE1bits.TXIE=0;
	PIR1bits.RCIF=0;
	IPR1bits.RCIP=1;       				// Set Receive of USART are High priority
	PIE1bits.RCIE=1;				// Enable RxD Interrupt
}

//************************************************
//* Function: InitializeAD                       *
//*        - Fosc/8 clock source                 *
//*        - Right justified                     *
//*        - 1 analog channels, 0 references     *
//*        - Channel 0                           *
//*        - Disable A/D interrupt               *
//************************************************
void InitializeAD(void)
{
	OpenADC(ADC_FOSC_8 & ADC_RIGHT_JUST & ADC_1ANA_0REF,
			ADC_CH0 & ADC_INT_OFF);
}
//************************************************
//* Function: InitializePWM1                     *
//*   - Initialize the CCP1 Module as PWM output *
//*     for controlling motor speed              *
//*     (see p160 MPLAB-CXX Reference Guide)     *
//*        - Set the period to FFh               *
//*        - Make RC2 an output                  *
//*        - Configure CCP1 for PWM              *
//************************************************
void InitializePWM1(void)
{
	TRISCbits.TRISC2=0;	//* setup RC2 for PWM output
	OpenPWM1(0xff); 	//* set period to 0xFF
}


//************************************************
//* Function: InitializeTimer2                   *
//*   - Initialize Timer2 for use with CCP1 PWM  *
//*        - 1:16 Postscale                      *
//*        - 1:4 Prescale                        *
//*        - Enable the timer                    *
//************************************************
void InitializeTMR2(void)
{
	OpenTimer2(T2_PS_1_4 & T2_POST_1_16);
}
//************************************************
//* Function: InitializePORT                     *
//*   - Make PORTD all outputs                   *
//*   - Turn 0ff all the LED's                   *
//************************************************
void InitializePORT(void)
{
	TRISD=0;
	PORTD=0x00;
}

	



